@@ -3,12 +3,93 @@ require 'sidekiq'
 
 module Sidekiq
   class Stats
+    def initialize
+      fetch_stats!
+    end
+
     def processed
-      Sidekiq.redis { |conn| conn.get("stat:processed") }.to_i
+      stat :processed
     end
 
     def failed
-      Sidekiq.redis { |conn| conn.get("stat:failed") }.to_i
+      stat :failed
+    end
+
+    def scheduled_size
+      stat :scheduled_size
+    end
+
+    def retry_size
+      stat :retry_size
+    end
+
+    def dead_size
+      stat :dead_size
+    end
+
+    def enqueued
+      stat :enqueued
+    end
+
+    def processes_size
+      stat :processes_size
+    end
+
+    def workers_size
+      stat :workers_size
+    end
+
+    def default_queue_latency
+      stat :default_queue_latency
+    end
+
+    def queues
+      Sidekiq::Stats::Queues.new.lengths
+    end
+
+    def fetch_stats!
+      pipe1_res = Sidekiq.redis do |conn|
+        conn.pipelined do
+          conn.get('stat:processed'.freeze)
+          conn.get('stat:failed'.freeze)
+          conn.zcard('schedule'.freeze)
+          conn.zcard('retry'.freeze)
+          conn.zcard('dead'.freeze)
+          conn.scard('processes'.freeze)
+          conn.lrange('queue:default'.freeze, -1, -1)
+          conn.smembers('processes'.freeze)
+          conn.smembers('queues'.freeze)
+        end
+      end
+
+      pipe2_res = Sidekiq.redis do |conn|
+        conn.pipelined do
+          pipe1_res[7].each {|key| conn.hget(key, 'busy'.freeze) }
+          pipe1_res[8].each {|queue| conn.llen("queue:#{queue}") }
+        end
+      end
+
+      s = pipe1_res[7].size
+      workers_size = pipe2_res[0...s].map(&:to_i).inject(0, &:+)
+      enqueued     = pipe2_res[s..-1].map(&:to_i).inject(0, &:+)
+
+      default_queue_latency = if (entry = pipe1_res[6].first)
+                                Time.now.to_f - Sidekiq.load_json(entry)['enqueued_at'.freeze]
+                              else
+                                0
+                              end
+      @stats = {
+        processed:             pipe1_res[0].to_i,
+        failed:                pipe1_res[1].to_i,
+        scheduled_size:        pipe1_res[2],
+        retry_size:            pipe1_res[3],
+        dead_size:             pipe1_res[4],
+        processes_size:        pipe1_res[5],
+
+        default_queue_latency: default_queue_latency,
+        workers_size:          workers_size,
+        enqueued:              enqueued
+      }
     end
 
     def reset(*stats)
@@ -25,43 +106,35 @@ module Sidekiq
       end
     end
 
-    def queues
-      Sidekiq.redis do |conn|
-        queues = conn.smembers('queues')
+    private
 
-        lengths = conn.pipelined do
-          queues.each do |queue|
-            conn.llen("queue:#{queue}")
+    def stat(s)
+      @stats[s]
+    end
+
+    class Queues
+      def lengths
+        Sidekiq.redis do |conn|
+          queues = conn.smembers('queues'.freeze)
+
+          lengths = conn.pipelined do
+            queues.each do |queue|
+              conn.llen("queue:#{queue}")
+            end
           end
+
+          i = 0
+          array_of_arrays = queues.inject({}) do |memo, queue|
+            memo[queue] = lengths[i]
+            i += 1
+            memo
+          end.sort_by { |_, size| size }
+
+          Hash[array_of_arrays.reverse]
         end
-
-        i = 0
-        array_of_arrays = queues.inject({}) do |memo, queue|
-          memo[queue] = lengths[i]
-          i += 1
-          memo
-        end.sort_by { |_, size| size }
-
-        Hash[array_of_arrays.reverse]
       end
     end
 
-    def enqueued
-      queues.values.inject(&:+) || 0
-    end
-
-    def scheduled_size
-      Sidekiq.redis {|c| c.zcard('schedule') }
-    end
-
-    def retry_size
-      Sidekiq.redis {|c| c.zcard('retry') }
-    end
-
-    def dead_size
-      Sidekiq.redis {|c| c.zcard('dead') }
-    end
-
     class History
       def initialize(days_previous, start_date = nil)
         @days_previous = days_previous
@@ -92,8 +165,8 @@ module Sidekiq
         end
 
         Sidekiq.redis do |conn|
-          conn.mget(keys).each_with_index do |value, i|
-            stat_hash[dates[i].to_s] = value ? value.to_i : 0
+          conn.mget(keys).each_with_index do |value, idx|
+            stat_hash[dates[idx].to_s] = value ? value.to_i : 0
           end
         end
 
@@ -118,7 +191,7 @@ module Sidekiq
     include Enumerable
 
     def self.all
-      Sidekiq.redis {|c| c.smembers('queues') }.sort.map {|q| Sidekiq::Queue.new(q) }
+      Sidekiq.redis {|c| c.smembers('queues'.freeze) }.sort.map {|q| Sidekiq::Queue.new(q) }
     end
 
     attr_reader :name
@@ -145,7 +218,7 @@ module Sidekiq
       Time.now.to_f - Sidekiq.load_json(entry)['enqueued_at']
     end
 
-    def each(&block)
+    def each
       initial_size = size
       deleted_size = 0
       page = 0
@@ -160,21 +233,21 @@ module Sidekiq
         break if entries.empty?
         page += 1
         entries.each do |entry|
-          block.call Job.new(entry, @name)
+          yield Job.new(entry, @name)
         end
         deleted_size = initial_size - size
       end
     end
 
     def find_job(jid)
-      self.detect { |j| j.jid == jid }
+      detect { |j| j.jid == jid }
     end
 
     def clear
       Sidekiq.redis do |conn|
         conn.multi do
           conn.del(@rname)
-          conn.srem("queues", name)
+          conn.srem("queues".freeze, name)
         end
       end
     end
@@ -210,7 +283,7 @@ module Sidekiq
                      "#{target}.#{method}"
                    end
                  when "ActiveJob::QueueAdapters::SidekiqAdapter::JobWrapper"
-                   args[0]
+                   @item['wrapped'] || args[0]
                  else
                    klass
                  end
@@ -224,7 +297,7 @@ module Sidekiq
                     arg
                   end
                 when "ActiveJob::QueueAdapters::SidekiqAdapter::JobWrapper"
-                  args[1..-1]
+                  @item['wrapped'] ? args[0]["arguments"] : []
                 else
                   args
                 end
@@ -267,8 +340,8 @@ module Sidekiq
 
     def safe_load(content, default)
       begin
-        yield *YAML.load(content)
-      rescue ::ArgumentError => ex
+        yield(*YAML.load(content))
+      rescue => ex
         # #1761 in dev mode, it's possible to have jobs enqueued which haven't been loaded into
         # memory yet so the YAML can't be loaded.
         Sidekiq.logger.warn "Unable to load YAML: #{ex.message}" unless Sidekiq.options[:environment] == 'development'
@@ -311,7 +384,7 @@ module Sidekiq
       raise "Retry not available on jobs which have not failed" unless item["failed_at"]
       remove_job do |message|
         msg = Sidekiq.load_json(message)
-        msg['retry_count'] = msg['retry_count'] - 1
+        msg['retry_count'] -= 1
         Sidekiq::Client.push(msg)
       end
     end
@@ -326,8 +399,8 @@ module Sidekiq
         Sidekiq.redis do |conn|
           conn.multi do
             conn.zadd('dead', now, message)
-            conn.zremrangebyscore('dead', '-inf', now - DeadSet::TIMEOUT)
-            conn.zremrangebyrank('dead', 0, - DeadSet::MAX_JOBS)
+            conn.zremrangebyscore('dead', '-inf', now - DeadSet.timeout)
+            conn.zremrangebyrank('dead', 0, - DeadSet.max_jobs)
           end
         end
       end
@@ -382,7 +455,7 @@ module Sidekiq
     end
 
     def size
-      Sidekiq.redis {|c| c.zcard(name) }
+      Sidekiq.redis { |c| c.zcard(name) }
     end
 
     def clear
@@ -401,7 +474,7 @@ module Sidekiq
       end
     end
 
-    def each(&block)
+    def each
       initial_size = @_size
       offset_size = 0
       page = -1
@@ -411,12 +484,12 @@ module Sidekiq
         range_start = page * page_size + offset_size
         range_end   = page * page_size + offset_size + (page_size - 1)
         elements = Sidekiq.redis do |conn|
-          conn.zrange name, range_start, range_end, :with_scores => true
+          conn.zrange name, range_start, range_end, with_scores: true
         end
         break if elements.empty?
         page -= 1
         elements.each do |element, score|
-          block.call SortedEntry.new(self, score, element)
+          yield SortedEntry.new(self, score, element)
         end
         offset_size = initial_size - @_size
       end
@@ -520,9 +593,6 @@ module Sidekiq
   # Allows enumeration of dead jobs within Sidekiq.
   #
   class DeadSet < JobSet
-    TIMEOUT = 180 * 24 * 60 * 60 # 6 months
-    MAX_JOBS = 10_000
-
     def initialize
       super 'dead'
     end
@@ -532,6 +602,14 @@ module Sidekiq
         each(&:retry)
       end
     end
+
+    def self.max_jobs
+      Sidekiq.options[:dead_max_jobs]
+    end
+
+    def self.timeout
+      Sidekiq.options[:dead_timeout_in_seconds]
+    end
   end
 
   ##
@@ -573,7 +651,7 @@ module Sidekiq
       count
     end
 
-    def each(&block)
+    def each
       procs = Sidekiq.redis { |conn| conn.smembers('processes') }.sort
 
       Sidekiq.redis do |conn|
@@ -586,7 +664,7 @@ module Sidekiq
           end
         end
 
-        result.each_with_index do |(info, busy, at_s), i|
+        result.each do |info, busy, at_s|
           hash = Sidekiq.load_json(info)
           yield Process.new(hash.merge('busy' => busy.to_i, 'beat' => at_s.to_f))
         end
@@ -616,6 +694,7 @@ module Sidekiq
   #   'queues' => ['default', 'low'],
   #   'busy' => 10,
   #   'beat' => <last heartbeat>,
+  #   'identity' => <unique string identifying the process>,
   # }
   class Process
     def initialize(hash)
@@ -642,6 +721,10 @@ module Sidekiq
       signal('TERM')
     end
 
+    def dump_threads
+      signal('TTIN')
+    end
+
     private
 
     def signal(sig)
@@ -655,7 +738,7 @@ module Sidekiq
     end
 
     def identity
-      @id ||= "#{self['hostname']}:#{self['pid']}"
+      self['identity']
     end
   end
 
@@ -681,7 +764,7 @@ module Sidekiq
   class Workers
     include Enumerable
 
-    def each(&block)
+    def each
       Sidekiq.redis do |conn|
         procs = conn.smembers('processes')
         procs.sort.each do |key|
@@ -706,13 +789,15 @@ module Sidekiq
     def size
       Sidekiq.redis do |conn|
         procs = conn.smembers('processes')
-        return 0 if procs.empty?
-
-        conn.pipelined do
-          procs.each do |key|
-            conn.hget(key, 'busy')
-          end
-        end.map(&:to_i).inject(:+)
+        if procs.empty?
+          0
+        else
+          conn.pipelined do
+            procs.each do |key|
+              conn.hget(key, 'busy')
+            end
+          end.map(&:to_i).inject(:+)
+        end
       end
     end
   end
