@@ -1,4 +1,5 @@
-require 'helper'
+# encoding: utf-8
+require_relative 'helper'
 require 'sidekiq'
 require 'sidekiq/web'
 require 'rack/test'
@@ -29,6 +30,18 @@ class TestWeb < Sidekiq::Test
       end
     end
 
+    it 'can show text with any locales' do
+      rackenv = {'HTTP_ACCEPT_LANGUAGE' => 'ru,en'}
+      get '/', {}, rackenv
+      assert_match(/Панель управления/, last_response.body)
+      rackenv = {'HTTP_ACCEPT_LANGUAGE' => 'es,en'}
+      get '/', {}, rackenv
+      assert_match(/Panel de Control/, last_response.body)
+      rackenv = {'HTTP_ACCEPT_LANGUAGE' => 'en-us'}
+      get '/', {}, rackenv
+      assert_match(/Dashboard/, last_response.body)
+    end
+
     describe 'busy' do
 
       it 'can display workers' do
@@ -50,17 +63,23 @@ class TestWeb < Sidekiq::Test
       end
 
       it 'can quiet a process' do
-        assert_nil Sidekiq.redis { |c| c.lpop "host:pid-signals" }
-        post '/busy', 'quiet' => '1', 'hostname' => 'host', 'pid' => 'pid'
+        identity = 'identity'
+        signals_key = "#{identity}-signals"
+
+        assert_nil Sidekiq.redis { |c| c.lpop signals_key }
+        post '/busy', 'quiet' => '1', 'identity' => identity
         assert_equal 302, last_response.status
-        assert_equal 'USR1', Sidekiq.redis { |c| c.lpop "host:pid-signals" }
+        assert_equal 'USR1', Sidekiq.redis { |c| c.lpop signals_key }
       end
 
       it 'can stop a process' do
-        assert_nil Sidekiq.redis { |c| c.lpop "host:pid-signals" }
-        post '/busy', 'stop' => '1', 'hostname' => 'host', 'pid' => 'pid'
+        identity = 'identity'
+        signals_key = "#{identity}-signals"
+
+        assert_nil Sidekiq.redis { |c| c.lpop signals_key }
+        post '/busy', 'stop' => '1', 'identity' => identity
         assert_equal 302, last_response.status
-        assert_equal 'TERM', Sidekiq.redis { |c| c.lpop "host:pid-signals" }
+        assert_equal 'TERM', Sidekiq.redis { |c| c.lpop signals_key }
       end
     end
 
@@ -352,6 +371,14 @@ class TestWeb < Sidekiq::Test
       end
     end
 
+    describe 'dashboard/stats' do
+      it 'redirects to stats' do
+        get '/dashboard/stats'
+        assert_equal 302, last_response.status
+        assert_equal 'http://example.org/stats', last_response.header['Location']
+      end
+    end
+
     describe 'stats' do
       include Sidekiq::Util
 
@@ -359,12 +386,13 @@ class TestWeb < Sidekiq::Test
         Sidekiq.redis do |conn|
           conn.set("stat:processed", 5)
           conn.set("stat:failed", 2)
+          conn.sadd("queues", "default")
         end
         2.times { add_retry }
         3.times { add_scheduled }
         4.times { add_worker }
 
-        get '/dashboard/stats'
+        get '/stats'
         @response = Sidekiq.load_json(last_response.body)
       end
 
@@ -389,6 +417,10 @@ class TestWeb < Sidekiq::Test
           assert_equal 4, @response["sidekiq"]["busy"]
         end
 
+        it 'reports processes' do
+          assert_equal 1, @response["sidekiq"]["processes"]
+        end
+
         it 'reports retries' do
           assert_equal 2, @response["sidekiq"]["retries"]
         end
@@ -429,6 +461,30 @@ class TestWeb < Sidekiq::Test
       end
     end
 
+    describe 'stats/queues' do
+      include Sidekiq::Util
+
+      before do
+        Sidekiq.redis do |conn|
+          conn.set("stat:processed", 5)
+          conn.set("stat:failed", 2)
+          conn.sadd("queues", "default")
+          conn.sadd("queues", "queue2")
+        end
+        2.times { add_retry }
+        3.times { add_scheduled }
+        4.times { add_worker }
+
+        get '/stats/queues'
+        @response = Sidekiq.load_json(last_response.body)
+      end
+
+      it 'reports the queue depth' do
+        assert_equal 0, @response["default"]
+        assert_equal 0, @response["queue2"]
+      end
+    end
+
     describe 'dead jobs' do
       it 'shows empty index' do
         get 'morgue'
@@ -529,7 +585,7 @@ class TestWeb < Sidekiq::Test
       Sidekiq.redis do |conn|
         conn.multi do
           conn.sadd("processes", key)
-          conn.hmset(key, 'busy', 4)
+          conn.hmset(key, 'info', Sidekiq.dump_json('hostname' => 'foo', 'started_at' => Time.now.to_f, "queues" => []), 'at', Time.now.to_f, 'busy', 4)
           conn.hmset("#{key}:workers", Time.now.to_f, msg)
         end
       end
